@@ -4,22 +4,23 @@
 #include "fcl/globals.h"
 #include "fcl/layer.h"
 #include "fcl/neuron.h"
+#include "fcl/bandpass.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <assert.h>
-
+#include <vector>
 
 /** Main class of Feedforward Closed Loop Learning.
  * Create an instance of this class to do the
  * learning. It will create the whole network
- * with an input layer, hidden layers and an
+ * with an input layer,  layers and an
  * output layer. Learning is done iterative
  * by first setting the input values and errors
  * and then calling doStep().
  *
- * (C) 2017,2018, Bernd Porr <bernd@glasgowneuro.tech>
+ * (C) 2017,2018-2022, Bernd Porr <bernd@glasgowneuro.tech>
  * (C) 2017,2018, Paul Miller <paul@glasgowneuro.tech>
  *
  * GNU GENERAL PUBLIC LICENSE
@@ -32,35 +33,13 @@ public:
 	
 	/** Constructor: FCL without any filters
 	 * \param num_of_inputs Number of inputs in the input layer
-	 * \param num_of_hidden_neurons_per_layer_array Number of neurons in each layer
-	 * \param _num_hid_layers Number of hidden layer (needs to match with array above)
-	 * \param num_outputs Number of output in the output layer
+	 * \param num_of_neurons_per_layer_array Number of neurons in each layer
+	 * \param _num_layers Number of  layer (needs to match with array above)
 	 **/
 	FeedforwardClosedloopLearning(
-			int num_of_inputs,
-			int* num_of_hidden_neurons_per_layer_array,
-			int _num_hid_layers,
-			int num_outputs);
-
-	/** Constructor: FCL with filters for both the input and hidden layers
-	 * \param num_of_inputs Number of inputs in the input layer
-         * \param num_of_hidden_neurons_per_layer_array Number of neurons in each layer
-	 * \param _num_hid_layers Number of hidden layer (needs to match with array above)
-	 * \param num_outputs Bumber of output in the output layer
-         * \param num_filtersInput Number of filters at the input layer, 0 = no filterbank
-         * \param num_filtersHidden Number of filters in the hiddel layers (usually zero)
-         * \param _minT Minimum/first temporal duration of the 1st filter
-         * \param _maxT Maximum/last temporal duration of the last filter
-         **/
-	FeedforwardClosedloopLearning(
-			int num_of_inputs,
-			int* num_of_hidden_neurons_per_layer_array,
-			int _num_hid_layers,
-			int num_outputs,
-			int num_filtersInput,
-			int num_filtersHidden,
-			double _minT,
-			double _maxT);
+		const int num_input, 
+		const std::vector<int> &num_of_neurons_per_layer
+	);
 
 	/** Destructor
          * De-allocated any memory
@@ -71,18 +50,14 @@ public:
          * \param input Array with the input values
          * \param error Array of the error signals
          **/
-	void doStep(double* input, double* error);
-
-	/** Python wrapper function. Not public.
-         **/
-	void doStep(double* input, int n1, double* error, int n2);
+	void doStep(const std::vector<double> &input, const std::vector<double> &error);
 
 	/** Gets the output from one of the output neurons
          * \param index: The index number of the output neuron.
          * \return The output value of the output neuron.
          **/
 	double getOutput(int index) {
-		return layers[num_hid_layers]->getOutput(index);
+		return layers[n_neurons_per_layer.size()-1]->getOutput(index);
 	}
 
 	/** Sets globally the learning rate
@@ -108,9 +83,8 @@ public:
 	void setMomentum(double momentum);
 
 	/** Sets the activation function of the Neuron
-         * \param _activationFunction: See Neuron::ActivationFunction for the different options.
          **/
-	void setActivationFunction(Neuron::ActivationFunction _activationFunction);
+	void setActivationFunction(FCLNeuron::ActivationFunction _activationFunction);
 
 	/** Inits the weights in all layers
          * \param max Maximum value of the weights.
@@ -119,7 +93,7 @@ public:
          **/
 	void initWeights(double max = 0.001,
 			 int initBias = 1,
-			 Neuron::WeightInitMethod weightInitMethod = Neuron::MAX_OUTPUT_RANDOM);
+			 FCLNeuron::WeightInitMethod weightInitMethod = FCLNeuron::MAX_OUTPUT_RANDOM);
 
 	/** Seeds the random number generator
          * \param s An arbitratry number.
@@ -131,36 +105,31 @@ public:
          **/
 	void setBias(double _bias);
 
-	/** Returns the number of hidden layers
-         * \return Integer value of how many hidden layers exist (total number is +1).
-         **/
-	int getNumHidLayers() {return num_hid_layers;};
-
 	/** Gets the total number of layers
          * \return The total number of all layers.
          **/
-	int getNumLayers() {return num_hid_layers+1;};
+	int getNumLayers() {return (int)n_neurons_per_layer.size();};
 
 	/** Gets a pointer to a layer
          * \param i Index of the layer.
          * \return A pointer to a layer class.
          **/
-	Layer* getLayer(int i) {assert (i<=num_hid_layers); return layers[i];};
+	FCLLayer* getLayer(unsigned i) {assert (i<=n_neurons_per_layer.size()); return layers[i];};
 
 	/** Gets the output layer
          * \return A pointer to the output layer which is also a Layer class.
          **/
-	Layer* getOutputLayer() {return layers[num_hid_layers];};
+	FCLLayer* getOutputLayer() {return layers[n_neurons_per_layer.size()-1];};
+
+	/** Gets the number of inputs
+	 * \return The number of inputs
+	 **/
+	int getNumInputs() {return ni;}
 
 	/** Returns all Layers
          * \return Returns a two dimensional array of all layers.
          **/
-	Layer** getLayers() {return layers;};
-
-	/** Sets if the learning should be the derivative of the error in each neuron
-         * \param useIt: If one the derivative of the error signal is used for learning.
-         **/
-	void setUseDerivative(int useIt);
+	FCLLayer** getLayers() {return layers;};
 
 	/** Saves the whole network
          * \param name: filename
@@ -175,22 +144,14 @@ public:
 	
 
 private:
-
-	int ni;
-	int nh;
-	int no;
-	int* n_hidden;
-	int num_hid_layers;
-
-	int nfInput;
-	int nfHidden;
-	double minT,maxT;
+	unsigned ni;
+	std::vector<int> n_neurons_per_layer;
 
 	double learningRateDiscountFactor = 1;
 
 	long int step = 0;
 
-	Layer** layers;
+	FCLLayer** layers;
 
 	// should be called to relay layer index to the layer
 	void setDebugInfo();
@@ -199,5 +160,4 @@ private:
 	void setStep();
 
 };
-
 #endif
